@@ -4,9 +4,7 @@ This module defines the Entity class for handling file uploads to a remote serve
 The Entity class provides methods to manage and upload files, such as protein structure files, to a remote storage system using the DeepOrigin FilesClient.
 """
 
-from dataclasses import dataclass, field
-import os
-from typing import Optional
+from dataclasses import dataclass
 
 
 @dataclass
@@ -17,7 +15,14 @@ class Entity:
     This class manages the remote path and provides an upload method to ensure that the entity's file is uploaded to the remote storage if it does not already exist there. It uses the DeepOrigin FilesClient for remote file operations.
     """
 
-    _remote_path: Optional[str] = field(default=None, init=False)
+    def to_hash(self) -> str:
+        """computes a hash of the entity"""
+        raise NotImplementedError("to_hash needs to be implemented in the child class")
+
+    @property
+    def _remote_path(self) -> str:
+        """the base path for the entity on the remote server"""
+        return f"{self._remote_path_base}{self.to_hash()}{self._preferred_ext}"
 
     def upload(self):
         """Upload the entity to the remote server."""
@@ -25,14 +30,13 @@ class Entity:
         from deeporigin.platform import file_api
 
         remote_files = file_api.get_object_directory(
-            file_path="/entities/",
+            file_path=self._remote_path_base,
             recursive=True,
         )
         remote_files = [file.Key for file in remote_files]
 
-        remote_path = self._remote_path_base + os.path.basename(self.file_path)
-        if remote_path not in remote_files:
-            file_api.upload_file(remote_path=remote_path, local_path=self.file_path)
-
-        # set protein path
-        self._remote_path = remote_path
+        if self._remote_path not in remote_files:
+            file_api.upload_file(
+                remote_path=self._remote_path,
+                local_path=self.file_path,
+            )
