@@ -5,47 +5,11 @@ This module encapsulates the Protein class, which is responsible for managing an
 protein structures in computational biology workflows. It provides functionalities to load protein data
 from various sources, preprocess structures, handle ligands, and visualize protein-ligand interactions.
 
-Key Features:
-- Protein Class: Manages protein data loaded from PDB IDs, file paths, or direct block content.
-  Handles structure selection, filtering of HETATM records, and extraction of metal and cofactor residues.
-- Initialization: Allows initializing a Protein instance using a PDB ID, local file, or block content,
-  ensuring only one source is provided and properly loading the structure.
-- Preparation Methods: Includes methods like `prepare`, `remove_hetatm`, `remove_resnames`, and
-  `remove_water` to preprocess and clean protein structures for downstream analysis.
-- Ligand Management: Provides methods to select and manage ligands within the protein structure,
-  enabling extraction and manipulation of ligand residues.
-- Visualization: Integrates with the ProteinViewer to render interactive visualizations of protein
-  structures within Jupyter Notebooks, facilitating intuitive analysis and presentation of protein-ligand complexes.
-- Utility Functions: Offers static and internal helper methods for downloading protein data, loading structures,
-  and creating new Protein instances with modified structures.
-
-Dependencies:
-- Biotite for structure handling
-- RDKit for cheminformatics
-- Plotly for interactive visualizations
-- External tools for enhanced functionality
-
-Usage Example:
-
-# Initialize with a PDB ID
-protein = Protein.from_pdb_id("1ABC")
-
-# Initialize with a file
-protein = Protein.from_file("/path/to/protein.pdb")
-
-# Select a specific chain
-chain_a = protein.select_chain('A')
-
-# Remove water molecules
-protein_no_water = protein.remove_water()
-
-# Visualize the protein structure
-protein.show()
-
 """
 
 from collections import defaultdict
 from dataclasses import dataclass, field
+import hashlib
 import io
 import os
 from pathlib import Path
@@ -59,16 +23,20 @@ from biotite.structure.io.pdb import PDBFile
 from deeporigin_molstar import DockingViewer, JupyterViewer, ProteinViewer
 import numpy as np
 
-from deeporigin.drug_discovery.constants import METAL_ELEMENTS, METALS, STATE_DUMP_PATH
+from deeporigin.drug_discovery.constants import (
+    METAL_ELEMENTS,
+    METALS,
+    PROTEINS_DIR,
+    STATE_DUMP_PATH,
+)
 from deeporigin.drug_discovery.external_tools.utils import (
     generate_html_output,
     get_protein_info_dict,
 )
 from deeporigin.exceptions import DeepOriginException
-from deeporigin.functions.pocket_finder import find_pockets
 
 from .entity import Entity
-from .ligand import Ligand
+from .ligand import Ligand, LigandSet
 from .pocket import Pocket
 
 
@@ -87,6 +55,7 @@ class Protein(Entity):
     block_content: Optional[str] = None
 
     _remote_path_base = "entities/proteins/"
+    _preferred_ext = ".pdb"
 
     @classmethod
     def from_name(cls, name: str) -> "Protein":
@@ -247,17 +216,17 @@ class Protein(Entity):
 
         return sequences
 
-    def model_loops(self) -> None:
+    def model_loops(self, use_cache: bool = True) -> None:
         """model loops in protein structure"""
 
-        from deeporigin.functions.loop_modelling import model_loops
+        from deeporigin.functions.loop_modelling import model_loops as _model_loops
 
         pdb_id = self.pdb_id
 
         if pdb_id is None:
             raise ValueError("Currently, PDB ID is required to model loops.")
 
-        file_path = model_loops(pdb_id=pdb_id)
+        file_path = _model_loops(pdb_id=pdb_id, use_cache=use_cache)
         protein = Protein.from_file(file_path)
         self.structure = protein.structure
 
@@ -265,84 +234,98 @@ class Protein(Entity):
     def dock(
         self,
         *,
-        ligand: Ligand,
+        ligand: Optional[Ligand] = None,
+        ligands: Optional[LigandSet | list[Ligand]] = None,
         pocket: Pocket,
-    ) -> str:
+        use_cache: bool = True,
+        reference_pose: Optional[Ligand] = None,
+    ):
         """Dock a ligand into a specific pocket of the protein.
 
-        This method performs molecular docking of a ligand into a specified pocket
-        of the protein structure. It uses the Deep Origin docking to
-        generate a 3D structure of the docked ligand.
+        This method performs docking of a ligand or a set of ligands into a specified pocket
+        of the protein structure.
 
         Args:
             ligand (Ligand): The ligand to dock into the protein pocket.
+            ligands (LigandSet | list[Ligand]): A set of ligands to dock into the protein pocket.
             pocket (Pocket): The specific pocket in the protein where the ligand
                 should be docked.
+            use_cache (bool): Whether to use cached results if available. Defaults to True.
+            reference_pose (Ligand): A reference pose to use for constrained docking. If provided, the constraints will be computed using the MCS of the reference pose and the all the ligands to dock.
 
-        Returns:
-            str: Path to the SDF file containing the docked ligand structure.
         """
-        from deeporigin.functions import docking
 
-        docked_ligand_sdf_file = docking.dock(
-            protein=self,
-            ligand=ligand,
-            pocket=pocket,
-        )
+        if ligands is None and ligand is None:
+            raise DeepOriginException(
+                "Either ligand or ligands must be provided to protein.dock()"
+            ) from None
 
-        return docked_ligand_sdf_file
+        if ligand is not None and ligands is None:
+            ligands = [ligand]
+
+        if isinstance(ligands, list):
+            ligands = LigandSet(ligands)
+
+        if reference_pose is not None:
+            # perform constrained docking
+
+            all_ligands = LigandSet([reference_pose] + ligands)
+
+            # find the mcs across all ligands
+            mcs_mol = all_ligands.mcs()
+
+            # compute constraints for each ligand
+            constraints = ligands.compute_constraints(
+                reference=reference_pose,
+                mcs_mol=mcs_mol,
+            )
+
+            # construct args
+            args = [
+                {
+                    "protein": self,
+                    "ligand": ligand,
+                    "pocket": pocket,
+                    "constraints": constraint,
+                    "use_cache": use_cache,
+                }
+                for ligand, constraint in zip(ligands, constraints, strict=True)
+            ]
+
+            from deeporigin.functions.docking import constrained_dock
+
+            # running in series for now, while we sort out the parallelization
+            all_top_poses = []
+            for arg in args:
+                _, _, top_pose = constrained_dock(**arg)
+                all_top_poses.append(top_pose)
+
+            return LigandSet.from_sdf_files(all_top_poses)
+        else:
+            # perform normal docking
+
+            # Import here to avoid circular import
+            from deeporigin.functions.docking import dock
+            from deeporigin.functions.parallel import run_func_in_parallel
+
+            args = [
+                {
+                    "protein": self,
+                    "ligand": ligand,
+                    "pocket": pocket,
+                    "use_cache": use_cache,
+                }
+                for ligand in ligands
+            ]
+
+            data = run_func_in_parallel(func=dock, args=args)
+
+            return LigandSet.from_sdf_files(data["results"])
 
     @property
     def coordinates(self):
         return self.structure.coord
 
-    # @beartype
-    # def prepare(self, model_loops: bool = False, pdb_id: str = "") -> "Protein":
-    #     """
-    #     Prepares the protein by calling the 'prepare' function with the specified protein path, PDB ID, and extension.
-    #     It extracts metal and cofactor residue names from the protein structure and passes them to the 'prepare' function.
-
-    #     Returns:
-    #         protein (Protein): The prepared Protein object.
-    #     Raises:
-    #         Exception: If the preparation of the protein fails.
-    #     """
-    #     pdb_id = pdb_id if pdb_id else self.pdb_id
-    #     if model_loops and not pdb_id:
-    #         raise ValueError("PDB ID must be provided to model loops.")
-
-    #     metal_resnames, cofactor_resnames = self.extract_metals_and_cofactors()
-    #     metals_to_keep = [
-    #         resname for resname in metal_resnames if resname.upper() in METALS
-    #     ]
-
-    #     response = prepare(
-    #         protein_path=self.file_path,
-    #         protein_pdb_id=pdb_id,
-    #         protein_extension=self.block_type,
-    #         metal_resnames=metals_to_keep,
-    #         cofactor_resnames=cofactor_resnames,
-    #         model_loops=model_loops,
-    #     )
-    #     if not response["prepared_protein_content"]:
-    #         raise Exception("Failed to prepare protein.")
-
-    #     protein_dir = Path(self.file_path).parent
-    #     base_name = (
-    #         Path(self.file_path).stem if self.file_path else "modified_structure"
-    #     )
-    #     new_file_name = protein_dir / f"{base_name}_prep.pdb"
-
-    #     intermediate_protein = Protein(
-    #         block_content=response["prepared_protein_content"], block_type="pdb"
-    #     )
-    #     intermediate_protein.write_to_file(str(new_file_name))
-
-    #     protein = Protein(file_path=new_file_name)
-    #     protein.pdb_id = self.pdb_id
-
-    #     return protein
-
     def _filter_hetatm_records(
         self, exclude_water: bool = True, keep_resnames: Optional[list[str]] = None
     ):
@@ -458,6 +441,7 @@ class Protein(Entity):
         self,
         pocket_count: int = 1,
         pocket_min_size: int = 30,
+        use_cache: bool = True,
     ) -> list[Pocket]:
         """Find potential binding pockets in the protein structure.
 
@@ -484,10 +468,15 @@ class Protein(Entity):
             >>> for pocket in pockets:
             ...     print(f"Pocket: {pocket.name}, Volume: {pocket.properties.get('volume')} Å³")
         """
-        results_dir = find_pockets(
-            self.file_path,
+        # Import here to avoid circular import
+        # note that name is changed to avoid conflict with the function
+        from deeporigin.functions.pocket_finder import find_pockets as _find_pockets
+
+        results_dir = _find_pockets(
+            protein=self,
             pocket_count=pocket_count,
             pocket_min_size=pocket_min_size,
+            use_cache=use_cache,
         )
 
         return Pocket.from_pocket_finder_results(results_dir)
@@ -619,7 +608,7 @@ class Protein(Entity):
 
         metal_resnames = set()
         cofactor_resnames = set()
-        for _, atoms in residue_groups.items():
+        for atoms in residue_groups.values():
             res_name = atoms[0].res_name.strip().upper()
             is_metal = all(
                 atom.element.strip().upper() in METAL_ELEMENTS for atom in atoms
@@ -634,15 +623,17 @@ class Protein(Entity):
 
         return metal_resnames, cofactor_resnames
 
-    def extract_ligand(self, exclude_resnames: Optional[set[str]] = None):
+    def extract_ligand(self, exclude_resnames: Optional[set[str]] = None) -> Ligand:
         """
-        Extracts ligand(s) from a PDB file using RDKit.
+        Extracts ligand(s) from a Protein object and removes them from the protein structure.
+        This method mutates the protein object by removing ligand records.
 
         Args:
             exclude_resnames (set): Residue names to exclude (e.g., water).
 
+        Returns:
+            Ligand: The extracted ligand molecule.
         """
-
         from rdkit import Chem
 
         if exclude_resnames is None:
@@ -651,6 +642,7 @@ class Protein(Entity):
         ligand_lines = []
         conect_lines = []
         ligand_resnames = set()
+        ligand_atom_serials = set()
 
         # First pass: collect HETATM lines and their residue names
         with open(self.file_path, "r") as f:
@@ -665,6 +657,12 @@ class Protein(Entity):
                         continue
                     ligand_lines.append(line)
                     ligand_resnames.add(resname)
+                    # Store atom serial for later removal from structure
+                    try:
+                        atom_serial = int(line[6:11].strip())
+                        ligand_atom_serials.add(atom_serial)
+                    except ValueError:
+                        continue
 
         # Second pass: collect CONECT records for the ligand atoms
         with open(self.file_path, "r") as f:
@@ -694,8 +692,124 @@ class Protein(Entity):
         if mol is None:
             raise ValueError("RDKit could not parse the ligand from the PDB block.")
 
+        # Now remove the ligand from the protein structure
+        self._remove_ligand_from_structure(ligand_atom_serials, ligand_resnames)
+
         return Ligand.from_rdkit_mol(mol)
 
+    def _remove_ligand_from_structure(
+        self, ligand_atom_serials: set[int], ligand_resnames: set[str]
+    ):
+        """
+        Remove ligand atoms from the protein structure and update block_content.
+
+        Args:
+            ligand_atom_serials: Set of atom serial numbers to remove
+            ligand_resnames: Set of residue names to remove
+        """
+        if not self.block_content:
+            return
+
+        # Filter out ligand lines from block_content
+        filtered_lines = []
+        lines = self.block_content.split("\n")
+        removed_atoms = 0
+        removed_conect = 0
+
+        for line in lines:
+            # Skip HETATM lines for ligands
+            if line.startswith("HETATM"):
+                resname = line[17:20].strip()
+                if resname in ligand_resnames:
+                    removed_atoms += 1
+                    continue
+
+            # Skip CONECT lines involving ligand atoms
+            if line.startswith("CONECT"):
+                try:
+                    atom1 = int(line[6:11].strip())
+                    if atom1 in ligand_atom_serials:
+                        removed_conect += 1
+                        continue
+                    # Check if any other atoms in CONECT are ligand atoms
+                    atom2 = int(line[11:16].strip()) if len(line) > 16 else None
+                    atom3 = int(line[16:21].strip()) if len(line) > 21 else None
+                    atom4 = int(line[21:26].strip()) if len(line) > 26 else None
+
+                    if (
+                        atom2
+                        and atom2 in ligand_atom_serials
+                        or atom3
+                        and atom3 in ligand_atom_serials
+                        or atom4
+                        and atom4 in ligand_atom_serials
+                    ):
+                        removed_conect += 1
+                        continue
+                except ValueError:
+                    # Keep malformed CONECT records
+                    pass
+
+            filtered_lines.append(line)
+
+        # Update MASTER record if it exists
+        for i, line in enumerate(filtered_lines):
+            if line.startswith("MASTER"):
+                # MASTER record format: MASTER xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx xxxxx
+                # Field 9 (index 8) is total number of atoms, Field 11 (index 10) is total number of CONECT records
+                parts = line.split()
+                if len(parts) >= 12:
+                    try:
+                        # Update atom count (field 9)
+                        old_atom_count = int(parts[8])
+                        new_atom_count = old_atom_count - removed_atoms
+                        parts[8] = str(new_atom_count)
+
+                        # Update CONECT count (field 11)
+                        old_conect_count = int(parts[10])
+                        new_conect_count = old_conect_count - removed_conect
+                        parts[10] = str(new_conect_count)
+
+                        # Reconstruct the MASTER line with proper spacing
+                        filtered_lines[i] = (
+                            f"{parts[0]:<6}{parts[1]:>5}{parts[2]:>5}{parts[3]:>5}{parts[4]:>5}{parts[5]:>5}{parts[6]:>5}{parts[7]:>5}{parts[8]:>5}{parts[9]:>5}{parts[10]:>5}{parts[11]:>5}"
+                        )
+                    except (ValueError, IndexError):
+                        # If we can't parse the MASTER record, leave it unchanged
+                        pass
+                break
+
+        # Update block_content
+        self.block_content = "\n".join(filtered_lines)
+
+        # Update the structure by reloading from the filtered content
+        try:
+            new_structure = self.load_structure_from_block(
+                self.block_content, self.block_type
+            )
+            # Ensure we get an AtomArray, not AtomArrayStack
+            if (
+                hasattr(new_structure, "array_length")
+                and new_structure.array_length() > 0
+            ):
+                # It's an AtomArrayStack, select the first structure
+                self.structure = new_structure[0]
+            else:
+                # It's already an AtomArray
+                self.structure = new_structure
+
+            # Update atom_types if they exist
+            if hasattr(self.structure, "atom_name"):
+                self.atom_types = self.structure.atom_name
+        except Exception as e:
+            # If structure reloading fails, log warning but don't fail the extraction
+            import warnings
+
+            warnings.warn(
+                f"Failed to update protein structure after ligand removal: {e}",
+                stacklevel=2,
+            )
+
     def _create_new_protein_with_structure(
         self, new_structure, suffix: str = "_modified"
     ) -> "Protein":
@@ -733,7 +847,8 @@ class Protein(Entity):
                 f"Failed to create new Protein with modified structure: {str(e)}"
             ) from e
 
-    def to_pdb(self, file_path: str):
+    @beartype
+    def to_pdb(self, file_path: Optional[str | Path] = None) -> str:
         """
         Write the protein structure to a PDB file.
 
@@ -742,10 +857,15 @@ class Protein(Entity):
 
 
         """
+
+        if file_path is None:
+            file_path = PROTEINS_DIR / (self.to_hash() + ".pdb")
+
         try:
             pdb_file = PDBFile()
             pdb_file.set_structure(self.structure)
-            pdb_file.write(file_path)
+            pdb_file.write(str(file_path))
+            return str(file_path)
         except Exception as e:
             raise RuntimeError(
                 f"Failed to write structure to file {file_path}: {str(e)}"
@@ -784,6 +904,37 @@ class Protein(Entity):
             if os.path.exists(temp_file_path):
                 os.remove(temp_file_path)
 
+    @beartype
+    def to_hash(self) -> str:
+        """Convert the protein to SHA256 hash of the PDB file content.
+
+        Returns:
+            str: SHA256 hash string of the PDB file content
+        """
+        import tempfile
+
+        # Create a temporary PDB file
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".pdb", delete=False
+        ) as temp_file:
+            temp_file_path = temp_file.name
+
+        try:
+            # Write the protein to the temporary file
+            self.to_pdb(temp_file_path)
+
+            # Read the file and compute SHA256 hash
+            with open(temp_file_path, "rb") as f:
+                pdb_content = f.read()
+                hash_object = hashlib.sha256(pdb_content)
+                hash_hex = hash_object.hexdigest()
+
+            return hash_hex
+        finally:
+            # Clean up the temporary file
+            if os.path.exists(temp_file_path):
+                os.remove(temp_file_path)
+
     @classmethod
     @beartype
     def from_base64(
@@ -856,8 +1007,12 @@ class Protein(Entity):
     @beartype
     def show(
         self,
+        *,
         pockets: Optional[list[Pocket]] = None,
         sdf_file: Optional[str] = None,
+        ligand: Optional[Ligand] = None,
+        ligands: Optional[LigandSet | list[Ligand]] = None,
+        poses: Optional[LigandSet | list[Ligand]] = None,
     ):
         """Visualize the protein structure in a Jupyter notebook using MolStar viewer.
 
@@ -885,7 +1040,34 @@ class Protein(Entity):
 
         current_protein_file = self._dump_state()
 
+        # poses is an alias for ligands
+        if poses is not None:
+            ligands = poses
+
+        if ligand is not None and ligands is not None:
+            raise DeepOriginException(
+                "Either ligand or ligands must be provided, not both"
+            ) from None
+
+        if ligand is not None and sdf_file is not None:
+            raise DeepOriginException(
+                "Either ligand or sdf_file must be provided, not both"
+            ) from None
+
+        if ligands is not None and sdf_file is not None:
+            raise DeepOriginException(
+                "Either ligands or sdf_file must be provided, not both"
+            ) from None
+
+        if ligand is not None:
+            sdf_file = ligand.to_sdf()
+        elif ligands is not None:
+            if isinstance(ligands, list):
+                ligands = LigandSet(ligands)
+            sdf_file = ligands.to_sdf()
+
         if pockets is None and sdf_file is None:
+            # we're only showing the protein, use ProteinViewer
             protein_viewer = ProteinViewer(
                 data=current_protein_file,
                 format="pdb",
@@ -893,6 +1075,7 @@ class Protein(Entity):
             html_content = protein_viewer.render_protein()
             JupyterViewer.visualize(html_content)
         elif pockets is not None and sdf_file is None:
+            # need to show pockets
             pocket_surface_alpha: float = 0.7
             protein_surface_alpha: float = 0.1
 
