@@ -18,8 +18,7 @@ import nest_asyncio
 import pandas as pd
 
 from deeporigin.drug_discovery.constants import tool_mapper
-from deeporigin.platform import tools_api
-from deeporigin.platform.utils import PlatformClients
+from deeporigin.platform import Client, tools_api
 from deeporigin.tools import job_viz_functions
 from deeporigin.utils.core import elapsed_minutes
 
@@ -63,7 +62,7 @@ class Job:
     _tool: dict = field(default_factory=dict)
 
     # clients
-    _platform_clients: Optional[PlatformClients] = None
+    client: Optional[Client] = None
 
     def __post_init__(self):
         self.sync()
@@ -90,7 +89,7 @@ class Job:
         cls,
         ids: list[str],
         *,
-        _platform_clients: Optional[PlatformClients] = None,
+        client: Optional[Client] = None,
     ) -> "Job":
         """Create a Job instance from a list of IDs.
 
@@ -103,7 +102,7 @@ class Job:
         return cls(
             name="job",
             _ids=ids,
-            _platform_clients=_platform_clients,
+            client=client,
         )
 
     @classmethod
@@ -111,7 +110,7 @@ class Job:
         cls,
         id: str,
         *,
-        _platform_clients: Optional[PlatformClients] = None,
+        client: Optional[Client] = None,
     ) -> "Job":
         """Create a Job instance from a single ID.
 
@@ -124,7 +123,7 @@ class Job:
         return cls(
             name="job",
             _ids=[id],
-            _platform_clients=_platform_clients,
+            client=client,
         )
 
     def sync(self):
@@ -135,15 +134,10 @@ class Job:
         reached a terminal state (Succeeded or Failed).
         """
 
-        # get the org_key from the platform clients if it's not None
-        org_key = getattr(self._platform_clients, "org_key", None)
-        tools_client = getattr(self._platform_clients, "ToolsApi", None)
-
         # use
         results = tools_api.get_statuses_and_progress(
             self._ids,
-            client=tools_client,
-            org_key=org_key,
+            client=self.client,
         )
 
         self._attributes = results
@@ -363,13 +357,9 @@ class Job:
             The result of the cancellation operation from utils.cancel_runs.
         """
 
-        org_key = getattr(self._platform_clients, "org_key", None)
-        tools_client = getattr(self._platform_clients, "ToolsApi", None)
-
         tools_api.cancel_runs(
             self._ids,
-            client=tools_client,
-            org_key=org_key,
+            client=self.client,
         )
 
 
@@ -382,7 +372,7 @@ def get_dataframe(
     include_inputs: bool = False,
     include_outputs: bool = False,
     resolve_user_names: bool = False,
-    _platform_clients: Optional[PlatformClients] = None,
+    client: Optional[Client] = None,
 ) -> pd.DataFrame:
     """Get a dataframe of the job statuses and progress reports.
 
@@ -418,30 +408,17 @@ def get_dataframe(
             },
         }
 
-    if _platform_clients is None:
-        from deeporigin.config import get_value
-
-        org_key = get_value()["organization_id"]
-    else:
-        org_key = _platform_clients.org_key
-
-    tools_client = getattr(_platform_clients, "ToolsApi", None)
-
     jobs = tools_api.get_tool_executions(
-        org_key=org_key,
         filter=_filter,
         page_size=10000,
-        client=tools_client,
+        client=client,
     )
 
     if resolve_user_names:
         from deeporigin.platform import entities_api
 
-        orgs_client = getattr(_platform_clients, "EntitiesApi", None)
-
         users = entities_api.get_organization_users(
-            org_key=org_key,
-            client=orgs_client,
+            client=client,
         )
 
         # Create a mapping of user IDs to user names
@@ -474,26 +451,24 @@ def get_dataframe(
         data["user_outputs"] = []
 
     for job in jobs:
-        attributes = job["attributes"]
-
         # Add basic fields
         data["id"].append(job["id"])
-        data["created_at"].append(attributes["createdAt"])
-        data["execution_id"].append(attributes["executionId"])
-        data["completed_at"].append(attributes["completedAt"])
-        data["started_at"].append(attributes["startedAt"])
-        data["status"].append(attributes["status"])
-        data["tool_key"].append(attributes["tool"]["key"])
-        data["tool_version"].append(attributes["tool"]["version"])
+        data["created_at"].append(job["createdAt"])
+        data["execution_id"].append(job["executionId"])
+        data["completed_at"].append(job["completedAt"])
+        data["started_at"].append(job["startedAt"])
+        data["status"].append(job["status"])
+        data["tool_key"].append(job["tool"]["key"])
+        data["tool_version"].append(job["tool"]["version"])
 
-        user_id = attributes.get("createdBy", "Unknown")
+        user_id = job.get("createdBy", "Unknown")
 
         if resolve_user_names:
             data["user_name"].append(user_id_to_name.get(user_id, "Unknown"))
         else:
             data["user_name"].append(user_id)
 
-        user_inputs = attributes.get("userInputs", {})
+        user_inputs = job.get("userInputs", {})
 
         if include_inputs:
             data["user_inputs"].append(user_inputs)
@@ -504,12 +479,12 @@ def get_dataframe(
             data["n_ligands"].append(1)
 
         # Handle protein_id (may not exist or metadata may be None)
-        metadata = attributes.get("metadata")
+        metadata = job.get("metadata")
 
         # Calculate run duration in minutes and round to nearest integer
-        if attributes["completedAt"] and attributes["startedAt"]:
-            start = parser.isoparse(attributes["startedAt"])
-            end = parser.isoparse(attributes["completedAt"])
+        if job["completedAt"] and job["startedAt"]:
+            start = parser.isoparse(job["startedAt"])
+            end = parser.isoparse(job["completedAt"])
             duration = round((end - start).total_seconds() / 60)
             data["run_duration_minutes"].append(duration)
         else:
@@ -519,7 +494,7 @@ def get_dataframe(
             data["metadata"].append(metadata)
 
         if include_outputs:
-            data["user_outputs"].append(attributes.get("userOutputs", {}))
+            data["user_outputs"].append(job.get("userOutputs", {}))
 
     # Create DataFrame
     df = pd.DataFrame(data)
