@@ -50,3 +50,14 @@ class PREFIXES:
     DB = "_database"
     ROW = "_row"
     FOLDER = "_workspace"
+
+
+number = int | float
+
+ENVS = Literal["edge", "prod", "staging"]
+
+API_ENDPOINT = {
+    "prod": "https://api.deeporigin.io",
+    "staging": "https://api.staging.deeporigin.io",
+    "edge": "https://api.edge.deeporigin.io",
+}
